package main

import (
	"f5"
	"mt"
)

const (
	MT_SERVER_INFO = iota
	MT_MYSQL_CONF = iota
	MT_MAX
)

type MetaMgr struct {
	f5.MetaMgr
}

func (this *MetaMgr) Init() *MetaMgr {
	this.MetaMgr.Init()
	configDir := "../config/"
	if !f5.IsOnlineEnv() {
		configDir = "/root/pub/conf_test/admin/"
	}
	metaClasses := &[]f5.MetaClass{
		f5.MetaClass{
			PrimKey: "InstanceId",
			FileName: configDir + "admin.cluster.json",
			Idx: MT_SERVER_INFO,
			RawMeta: (*mt.ServerInfoMetas)(nil),
			WrapMeta: (*MtwServerInfo)(nil)},
		f5.MetaClass{
			FileName: configDir + "admin.mysql.json",
			Idx: MT_MYSQL_CONF,
			RawMeta: (*mt.MysqlConfMetas)(nil),
			WrapMeta: (*MtwMysqlConf)(nil)},
	}
	this.MetaMgr.RegisterMetaClasses(metaClasses)
	this.Load()
	return this
}

func (this *MetaMgr) UnInit() {
	this.MetaMgr.UnInit()
}

func (this *MetaMgr) GetServer(instance_id int32) *MtwServerInfo {
	v, ok := this.MetaMgr.GetMetaById(MT_SERVER_INFO, instance_id).(*MtwServerInfo)
	if ok {
		return v
	} else {
		return nil
	}
}

func (this *MetaMgr) GetCurrServer() *MtwServerInfo {
	return this.GetServer(int32(f5.App.GetInstanceId()))
}

func (this *MetaMgr) GetMysqlCluster() []*MtwMysqlConf {
	return this.GetMetaList(MT_MYSQL_CONF).([]*MtwMysqlConf)
}
