package main

import (
	"math/rand"
	"encoding/json"
	"database/sql"
	"q5"
	"f5"
)

type UserMgr struct {
}

func (this *UserMgr) Init() *UserMgr {
	G.HttpServer.RegisterRestHandle("/user/login", this._userLogin)

	G.HttpServer.DefineGroup("auth", this.auth)

	G.HttpServer.UseGroupBegin("auth")
	App.RegisterAuthedHandler("/user/info", this._userInfo)
	App.RegisterAuthedHandler("/user/logout", this._userLogout)
	G.HttpServer.UseGroupEnd()
	return this
}

func (this *UserMgr) UnInit() {

}

func (this *UserMgr) auth(c *f5.Context) {
	authStr := c.Header("Authorization")
	if len(authStr) < 8 {
		c.Abort()
		return
	}
	token := authStr[7:]
	f5.SysLog().Debug("auth token:%s", token)
	tmpStrings := q5.StrSplit(token, ".")
	if len(tmpStrings) < 4 {
		c.Abort()
		return
	}
	if userName, err := q5.Base64Decode(tmpStrings[1]); err == nil {
		md5Str := tmpStrings[2] + userName + tmpStrings[3] + SESSION_KEY
		sign := q5.Md5Str(md5Str)
		if sign != tmpStrings[0] {
			c.Abort()
			return
		}
		user := &AuthedUserImpl{}
		user.accountId = userName
		user.registerTime = q5.NewXString(tmpStrings[2]).GetInt64()
		c.Set("user", user)
		return
	} else {
		c.Abort()
		return
	}
}

func (this *UserMgr) _userLogin(c *f5.Context) {
	url := "https://mp.kingsome.cn/api/user/login"
	content := q5.NewMxoObject()
	reqObj := struct {
		UserName string `json:"username"`
		Password string `json:"password"`
	}{}
	postBody := c.GetBody().GetString()
	if err := json.Unmarshal([]byte(postBody), &reqObj); err != nil{
		c.ResponseErr(500, "服务器内部错误")
		return
	}
	f5.SysLog().Debug("boyd:%s", postBody)
	content.SetXValue("username", q5.NewXString(reqObj.UserName))
	content.SetXValue("password", q5.NewXString(reqObj.Password))
	respStr := ""
	if err := q5.HttpPostContent(url,
		"application/json;charset=utf-8",
		content.ToJsonStr(),
		&respStr); err != nil {
		c.ResponseErr(500, "服务器内部错误")
		return
	}
	f5.SysLog().Debug("respStr:%s", respStr)
	respObj := struct {
		ErrCode *int32 `json:"errcode"`
		ErrMsg string `json:"errmsg"`
		Token string `json:"token"`
		MpToken string `json:"mp_token"`
		UserInfo *struct {
			UserName string `json:"username"`
			FullName string `json:"fullname"`
			AvatarUrl string `json:"avatar"`
		} `json:"userInfo"`
	}{}
	if err := json.Unmarshal([]byte(respStr), &respObj); err != nil || respObj.ErrCode == nil {
		c.ResponseErr(500, "服务器内部错误")
		return
	}
	if *respObj.ErrCode != 0  {
		c.Response(respStr)
		return
	}
	userInfo := respObj.UserInfo
	nowUnix := f5.App.NowUnix()
	conn := G.DBEngine.NewAdminConn()
	row := conn.QueryRow("SELECT username, createtime FROM `user` WHERE username=?;",
		userInfo.UserName)
	{
		userName := ""
		var registerTime int64
		err := row.Scan(&userName, &registerTime)
		if err != nil && err != sql.ErrNoRows {
			c.ResponseErr(500, "服务器内部错误")
			return
		}
		if err == sql.ErrNoRows {
			//新用户
			conn.Exec("INSERT `user`(username, fullname, avatar_url, createtime, " +
				"  modifytime) " +
				"VALUES(?, ?, ?, ?, ?);",
				userInfo.UserName,
				userInfo.FullName,
				userInfo.AvatarUrl,
				nowUnix,
				nowUnix)
			registerTime = nowUnix
		} else {
			//老用户
			conn.Exec("UPDATE `user` SET fullname=?, avatar_url=?, " +
				"  modifytime=? " +
				"WHERE username=?;",
				userInfo.FullName,
				userInfo.AvatarUrl,
				nowUnix,
				userInfo.UserName)
		}
		randNum := nowUnix + int64(rand.Int31())
		md5Str := q5.NewXInt64(registerTime).GetString() +
			userName +
			q5.NewXInt64(randNum).GetString() +
			SESSION_KEY
		newToken := q5.Md5Str(md5Str)
		respObj.MpToken = respObj.Token
		respObj.Token = newToken + "." +
			q5.Base64Encode(userName) + "." +
			q5.NewXInt64(registerTime).GetString() + "." +
			q5.NewXInt64(randNum).GetString() + "." +
			q5.Base64Encode(respObj.Token)
		newResp, _ := json.Marshal(respObj);
		c.Response(string(newResp))
	}
}

func (this *UserMgr) _userInfo(c *f5.Context, user *AuthedUser) {
	authStr := c.Header("Authorization")
	if len(authStr) < 8 {
		c.ResponseErr(101, "验证失败")
		return
	}
	token := authStr[7:]
	f5.SysLog().Debug("auth token:%s", token)
	tmpStrings := q5.StrSplit(token, ".")
	if len(tmpStrings) < 5 {
		c.ResponseErr(101, "验证失败")
		return
	}
	if mpToken, err := q5.Base64Decode(tmpStrings[4]); err == nil {
		url := "https://mp.kingsome.cn/api/user/info"
		params := q5.NewMxoObject()
		respStr, err := q5.HttpGetEx(url,
			params.AsXObject(),
			func (setOpt func(int32, *q5.XValue, *q5.XValue)) {
				setOpt(q5.HTTP_OPT_ADD_HEADER,
					q5.NewXString("authorization"),
					q5.NewXString("Bearer " + mpToken))
			})
		if err == nil {
			f5.SysLog().Debug("respStr:%s", respStr)
			c.Response(respStr)
		} else {
			f5.SysLog().Debug("respStr:%s", err)
			c.ResponseErr(500, "服务器内部错误")
		}
	} else {
		c.ResponseErr(101, "验证失败")
		return
	}
}

func (this *UserMgr) _userLogout(c *f5.Context, user *AuthedUser) {

}
