const app = require('j7/app');
const utils = require('j7/utils');
const log = require('j7/log');

const metaClassList = [];
const metaClasses = {};

function registerMetaClass(fileName, primKey, wrapClass, aliseName = '') {
  const metaClass = {
    'fileName' : fileName,
    'primKey' : primKey,
    'wrapClassName': wrapClass,
    'wrapClass': require('./' + wrapClass),
    'rawList' : [],
    'rawHash' : {},
    'wrapList' : [],
    'wrapHash' : {},
    'aliseName': aliseName
  };
  metaClassList.push(metaClass);
  if (aliseName) {
    metaClasses[aliseName] = metaClass;
  } else {
    metaClasses[wrapClass] = metaClass;
  }
}

function onAllLoadPost(classList) {
  for (let i = 0; i < 3; ++i) {
    classList.forEach((metaClass) => {
      metaClass['wrapList'].forEach((item) => {
        if (item['_init' + i]) {
          item['_init' + i](exports);
        }
      });
    });
  }
  classList.forEach((metaClass) => {
    metaClass['wrapList'].forEach((item) => {
      item.lock();
    });
  });
}

function loadMeta(metaClass) {
  const json = utils.readJsonFromFile(metaClass['fileName']);
  if (!json) {
    throw new Error('读取配置' + metaClass['fileName'] + '失败');
  }
  if (Array.isArray(json)) {
    metaClass['rawList'] = json;

  } else {
    metaClass['rawList'].push(json);
  }
  let idx = 0;
  metaClass['rawList'].forEach(function (item) {
    const wrapOjb = new metaClass['wrapClass'](item, metaClass);
    const wrapProxy = new Proxy(wrapOjb, wrapOjb._getHandler());
    metaClass['wrapList'].push(wrapProxy);
    if (metaClass['primKey'] == '') {
      metaClass['rawHash'][idx] = item;
      metaClass['wrapHash'][idx] = wrapProxy;
    } else {
      metaClass['rawHash'][item[metaClass['primKey']]] = item;
      metaClass['wrapHash'][item[metaClass['primKey']]] = wrapProxy;
    }
    ++idx;
  });
}

function load() {
  metaClassList.forEach((metaClass) => {
    loadMeta(metaClass);
    //log.debug(utils.jsonEncode(metaClass));
  });
  onAllLoadPost(metaClassList);
}

function init() {
  let configDir = './config/';
  if (utils.isOnlineEnv()) {
    configDir = '../config/';
  } else if (utils.getArgv('env') == 'dev'){
    configDir = './config_dev/';
  }
  console.log(configDir, utils.getArgv('env'));

  let resDir = './res/';
  if (utils.isOnlineEnv()) {
    resDir = '../res/';
  } else if (utils.getArgv('env') == 'dev'){
    resDir = './res_dev/';
  }

  registerMetaClass(configDir + 'config.json',
                    '',
                    'Config'
                   );
}

function getMetaClass(name) {
  return utils.hasKey(metaClasses, name) ? metaClasses[name] : null;
}

function getMetaByKey(name, key) {
  const metaClass = getMetaClass(name);
  return metaClass && key in metaClass['wrapHash'] ? metaClass['wrapHash'][key] : null;
}

function getMetaList(name) {
  const metaClass = getMetaClass(name);
  return metaClass ? metaClass['wrapList'] : null;
}

function callMetaStatic(name, method, ...args) {
  const metaClass = getMetaClass(name);
  return metaClass['wrapClass'][method](...args);
}

function traverseMetaList(name, cb) {
  const metaList = getMetaList(name);
  if (metaList) {
    for (let i = 0; i < metaList.length; ++i) {
      if (!cb(metaList[i], i, metaList.length)) {
        break;
      }
    }
  }
}

exports.init = init;

exports.getMetaByKey = getMetaByKey;
exports.traverseMetaList = traverseMetaList;
exports.callMetaStatic = callMetaStatic;
