const assert = require('assert');
const ws = require('nodejs-websocket');
const app = require('j7/app');
const utils = require('j7/utils');
const log = require('j7/log');
const config = require('j7/config');
const http = require('j7/http');
const BaseService = require('./baseservice');
const metaFactory = require('../metadata/factory');

class WsServer extends BaseService {

  async init() {
    let data = '';
    const server = ws.createServer((socket) => {

      socket.on('text', (str) => {
        //console.log(str);
        data += str;
        //#0000000
        while (data.length >= 8) {
          if (data[0] != '#') {
            socket.close('error package');
            break;
          }
          const pkgLen = parseInt(data.slice(1, 8), 10);
          if (isNaN(pkgLen)) {
            socket.close('error pkgLen');
            break;
          }
          //console.log(data, pkgLen, data.slice(1,8));
          if (data.length >= 8 + pkgLen) {
            this.parsePackage(socket, data.slice(8, 8 + pkgLen));
            data = data.slice(8 + pkgLen);
            console.log(data);
          }
        }
      });

      socket.on('close', (code, reason) => {
        console.log('connection close', code, reason);
      });

    }).listen(config('ws_listen_port'));
  }

  async parsePackage(socket, data) {
    console.log('parsePackage', data);
    const msg = utils.jsonDecode(data);
    {
      const beginTick = utils.getTickCount();
      const {err, data} = await http.get(msg['url'], msg['params']);
      const endTick = utils.getTickCount();
      if (socket.readyState == socket.OPEN) {
        const rspText = utils.jsonEncode({
          'seqId': msg['seqId'],
          'payload': msg['payload'],
          'time_cost': endTick - beginTick,
          'err': err,
          'data': data
        });
        const text = '#' + utils.pad(rspText.length, 7) + rspText;
        console.log(text);
        socket.sendText(text);
      }
    }
  }

}

module.exports = WsServer;
