package main

import (
	"f5"
	"mt"
)

const (
	MT_SERVER_INFO = 0
	MT_GAME2005_MYSQL_CONF
	MT_GAME2005_MAIL
	MT_MAX
)

type MetaMgr struct {
	f5.MetaMgr
}

func (this *MetaMgr) Init() *MetaMgr {
	this.MetaMgr.Init()
	configDir := "../config/"
	resDir := "../res/"
	if !f5.IsOnlineEnv() {
		configDir = "/var/data/conf_test/mailman/"
		resDir = "/var/data/conf_test/mailman/res/"
	}
	metaClasses := &[]f5.MetaClass{
		f5.MetaClass{
			PrimKey: "InstanceId",
			FileName: configDir + "mailman.cluster.json",
			Idx: MT_SERVER_INFO,
			RawMeta: (*mt.ServerInfoMetas)(nil),
			WrapMeta: (*MtwServerInfo)(nil)},
		f5.MetaClass{
			PrimKey: "InstanceId",
			FileName: configDir + "game2005.mysql.cluster.json",
			Idx: MT_GAME2005_MYSQL_CONF,
			RawMeta: (*mt.Game2005MysqlConfMetas)(nil),
			WrapMeta: (*MtwGame2005MysqlConf)(nil)},
		f5.MetaClass{
			PrimKey: "InstanceId",
			FileName: resDir + "6000@game2005.mail.json",
			Idx: MT_GAME2005_MAIL,
			RawMeta: (*mt.Game2005MailMetas)(nil),
			WrapMeta: (*MtwGame2005Mail)(nil)},
	}
	this.MetaMgr.RegisterMetaClasses(metaClasses)
	this.Load()
	return this
}

func (this *MetaMgr) UnInit() {
	this.MetaMgr.UnInit()
}

func (this *MetaMgr) GetServer(instance_id int32) *MtwServerInfo {
	v, ok := this.MetaMgr.GetMetaById(MT_SERVER_INFO, instance_id).(*MtwServerInfo)
	if ok {
		return v
	} else {
		return nil
	}
}

func (this *MetaMgr) GetCurrServer() *MtwServerInfo {
	return this.GetServer(int32(f5.App.GetInstanceId()))
}

func (this *MetaMgr) GetGame2005MysqlCluster() []*MtwGame2005MysqlConf {
	return this.GetMetaList(MT_GAME2005_MYSQL_CONF).([]*MtwGame2005MysqlConf)
}

func (this *MetaMgr) GetGame2005Mail(mailId int32) *MtwGame2005Mail {
	v, ok := this.MetaMgr.GetMetaById(MT_GAME2005_MAIL,mailId).(*MtwGame2005Mail)
	if ok {
		return v
	} else {
		return nil
	}
}
