# -*- coding: utf-8 -*-
#!/usr/bin/python

import sys
sys.path.append('../local_packages')
import q7
import f7

import os
import json
import time
import urllib.parse
import urllib.request
import pymysql

CONFIG_DIR = '../config' if f7.isOnlineEnv() else '/var/data/conf_test/mailsender/config'
RES_DIR = '../res' if f7.isOnlineEnv() else '/var/data/conf_test/mailsender/res'

db_idx_hash = {}

def httpGet(url, params):
    param = urllib.parse.urlencode(params)
    url = url + param
    response = urllib.request.urlopen(url=url)
    data = response.read()
    print(url)
    print(str(data))
    return data

def getMailConf(mailid):
    mail_conf = json.loads(open(RES_DIR + '/6000@game2005.mail.json', 'r').read())
    for conf in mail_conf:
        if conf['mailid'] == mailid:
            return conf
    return None

def getMailAttachmentsByVip(mail_conf, vip_lv):
    items = mail_conf['attachments'].split('|')[vip_lv - 1]
    return items.replace('$', '|')

def getMailAttachmentsByPriv(mail_conf, privi_lv):
    items = mail_conf['attachments'].split('|')[privi_lv - 1]
    return items.replace('$', '|')

def sendVipLvUpMail(account_id, old_vip_lv, new_vip_lv):
    url = 'https://gamemail.kingsome.cn/webapp/index.php?' if f7.isOnlineEnv() else 'https://gamemail-test.kingsome.cn/webapp/index.php?'
    for vip_lv in range(old_vip_lv + 1, new_vip_lv + 1):
        mail_conf = getMailConf(1)
        params = {
            'c': 'MailMgr',
            'a': 'sendMail',
            'gameid': 2005,
            'to': account_id,
            'mailtype': 1,
            'mailsubtype': -2,
            'usertype': 2,
            'content': mail_conf['content'],
            'subject': mail_conf['subject'],
            'sendtime': time.time(),
            'expiretime': time.time() + 3600 * 24 * 30,
            'attachments': getMailAttachmentsByVip(mail_conf, vip_lv)
        }
        httpGet(url, params)

def sendPriviDailyMail(account_id, privi_lv):
    url = 'https://gamemail.kingsome.cn/webapp/index.php?' if f7.isOnlineEnv() else 'https://gamemail-test.kingsome.cn/webapp/index.php?'
    mail_conf = getMailConf(3)
    params = {
        'c': 'MailMgr',
        'a': 'sendMail',
        'gameid': 2005,
        'to': account_id,
        'mailtype': 1,
        'mailsubtype': -4,
        'usertype': 2,
        'content': mail_conf['content'],
        'subject': mail_conf['subject'],
        'sendtime': time.time(),
        'expiretime': time.time() + 3600 * 24 * 30,
        'attachments': getMailAttachmentsByPriv(mail_conf, privi_lv)
    }
    httpGet(url, params)

def sendVipWeeklyMail(account_id, vip_lv):
    url = 'https://gamemail.kingsome.cn/webapp/index.php?' if f7.isOnlineEnv() else 'https://gamemail-test.kingsome.cn/webapp/index.php?'
    mail_conf = getMailConf(2)
    params = {
        'c': 'MailMgr',
        'a': 'sendMail',
        'gameid': 2005,
        'to': account_id,
        'mailtype': 1,
        'mailsubtype': -3,
        'usertype': 2,
        'content': mail_conf['content'],
        'subject': mail_conf['subject'],
        'sendtime': time.time(),
        'expiretime': time.time() + 3600 * 24 * 30,
        'attachments': getMailAttachmentsByVip(mail_conf, vip_lv)
    }
    httpGet(url, params)

def fetchEvent():
    print('fetchEvent')
    global db_idx_hash
    mysql_cluster_conf = json.loads(open(CONFIG_DIR + '/mailsender.mysql.cluster.json', 'r').read())
    for conf in mysql_cluster_conf:
        print(conf)
        conn = pymysql.connect(host = conf['host'],
                               port = conf['port'],
                               user = conf['user'],
                               passwd = conf['passwd'],
                               db = conf['dbname'],
                               charset = 'utf8'
        )
        cursor = conn.cursor()
        last_idx = db_idx_hash.get(conf['instance_id'], 0)
        while True:
            cursor.execute('SELECT idx, sender_id, event_name, param1, param2 '
                           'FROM `event` '
                           'WHERE idx > %d AND status = 0 AND event_name = "vip_level_up" '
                           'LIMIT 0, 1000'
                           % (last_idx))
            handled_idx = []
            has_data = False
            for row in cursor:
                has_data = True
                print(row)
                last_idx = max(row[0], last_idx)
                account_id = row[1]
                old_vip_lv = int(row[3])
                new_vip_lv = int(row[4])
                sendVipLvUpMail(account_id, old_vip_lv, new_vip_lv)
                handled_idx.append(row[0])
                time.sleep(0.001);
            else:
                for idx in handled_idx:
                    cursor.execute('UPDATE `event` SET status = 1 WHERE idx = %d '
                                   % (idx))
                    print('UPDATE `event` SET status = 1 WHERE idx = %d '
                          % (idx))
                    conn.commit()
                if not has_data:
                    break
        #end while
        db_idx_hash[conf['instance_id']] = last_idx

def fetchEventTimerFunc(refresh_time):
    def done_callback():
        f7.timer.callLater(refresh_time,
                           lambda : fetchEventTimerFunc(refresh_time))
#    dailyMail()
    f7.app.createAsyncTask(done_callback, fetchEvent, ())

def dailyMail():
    print('dailyMail')
    mysql_cluster_conf = json.loads(open(CONFIG_DIR + '/mailsender.mysql.cluster.json', 'r').read())
    nowtime = time.time()
    for conf in mysql_cluster_conf:
        conn = pymysql.connect(host = conf['host'],
                               port = conf['port'],
                               user = conf['user'],
                               passwd = conf['passwd'],
                               db = conf['dbname'],
                               charset = 'utf8'
        )
        cursor = conn.cursor()
        last_idx = 0
        while True:
            cursor.execute('SELECT idx, accountid, vip_lv, privi_lv, createtime, modifytime, '
                           '  privi_daily_mail_lasttime, vip_weekly_mail_lasttime '
                           'FROM `vip_user` '
                           'WHERE idx > %d '
                           'LIMIT 0, 1000'
                           % (last_idx))
            handled_vip_idx = []
            handled_privi_idx = []
            has_data = False
            for row in cursor:
                has_data = True
                last_idx = max(row[0], last_idx)
                idx = row[0]
                account_id = row[1]
                vip_lv = row[2]
                privi_lv = row[3]
                createtime = row[4]
                modifytime = row[5]
                privi_daily_mail_lasttime = row[6]
                vip_weekly_mail_lasttime = row[7]
                if privi_daily_mail_lasttime <= 0 or \
                   q7.getDaySeconds(nowtime) > q7.getDaySeconds(privi_daily_mail_lasttime):
                    if privi_lv > 0:
                        sendPriviDailyMail(account_id, privi_lv)
                        handled_privi_idx.append(idx)
                if vip_weekly_mail_lasttime <= 0 or \
                   q7.getMondaySeconds(nowtime) > q7.getMondaySeconds(vip_weekly_mail_lasttime):
                    if vip_lv > 0:
                        sendVipWeeklyMail(account_id, vip_lv)
                        handled_vip_idx.append(idx)
                time.sleep(0.001);
            else:
                for idx in handled_privi_idx:
                    cursor.execute('UPDATE `vip_user` SET privi_daily_mail_lasttime = %d WHERE idx = %d '
                                   % (nowtime, idx))
                    conn.commit()
                for idx in handled_vip_idx:
                    cursor.execute('UPDATE `vip_user` SET vip_weekly_mail_lasttime = %d WHERE idx = %d '
                                   % (nowtime, idx))
                    conn.commit()
                if not has_data:
                    break
        #end while

def dailyMailTimerFunc(refresh_time):
    def done_callback():
        f7.timer.callAt(q7.getDaySeconds(time.time(), 1) + refresh_time,
                        lambda : dailyMailTimerFunc(refresh_time))
    f7.app.createAsyncTask(done_callback, dailyMail, ())

def getConf():
    cluster_conf = json.loads(open(CONFIG_DIR + '/mailsender.cluster.json', 'r').read())
    for conf in cluster_conf:
        if conf['instance_id'] == f7.app.getInstanceId():
            return conf
    return None

if __name__ == "__main__":
    q7.xPrint('pid %d' % os.getpid())
    f7.app.init('/data/logs/mailsender/logs')
    f7.udplog.info('mailsender start pid:%d node_id:%d instance_id:%d' % (
        os.getpid(),
        f7.app.getNodeId(),
        f7.app.getInstanceId())
    )

    conf = getConf()
#    f7.app.registerHandler('Login', 'auth', __loginAuth)
    conf['refresh_time'] = 10
    f7.timer.callLater(conf['refresh_time'],
                       lambda : fetchEventTimerFunc(conf['refresh_time']))
    conf['daily_mail_sendtime'] = 10
    f7.timer.callAt(q7.getDaySeconds(time.time(), 1) + conf['daily_mail_sendtime'],
                    lambda : dailyMailTimerFunc(conf['daily_mail_sendtime']))

    f7.app.listen(conf['listen_port'])
    f7.app.start()
