package main

import (
	"f5"
	"mt"
)

const (
	MT_SERVER_INFO = iota
	MT_RANK_CONFIG
	MT_MAX
)

type MetaMgr struct {
	f5.MetaMgr
}

func (this *MetaMgr) Init() *MetaMgr {
	this.MetaMgr.Init()
	configDir := "../config/"
	if !f5.IsOnlineEnv() {
		configDir = "/var/data/conf_test/rankserver/"
	}
	metaClasses := &[]f5.MetaClass{
		f5.MetaClass{
			PrimKey: "InstanceId",
			FileName: configDir + "rankserver.cluster.json",
			Idx: MT_SERVER_INFO,
			RawMeta: (*mt.ServerInfoMetas)(nil),
			WrapMeta: (*MtwServerInfo)(nil)},
		f5.MetaClass{
			PrimKey: "Name",
			FileName: configDir + "rank_config.json",
			Idx: MT_RANK_CONFIG,
			RawMeta: (*mt.RankConfig)(nil),
			WrapMeta: (*MtwRankConfig)(nil)},
	}
	this.MetaMgr.RegisterMetaClasses(metaClasses)
	this.Load()
	return this
}

func (this *MetaMgr) UnInit() {
	this.MetaMgr.UnInit()
}

func (this *MetaMgr) GetServer(instance_id int32) *MtwServerInfo {
	v, ok := this.MetaMgr.GetMetaById(MT_SERVER_INFO, instance_id).(*MtwServerInfo)
	if ok {
		return v
	} else {
		return nil
	}
}

func (this *MetaMgr) GetRankConfig() []*MtwRankConfig {
	return this.MetaMgr.GetMetaList(MT_RANK_CONFIG).([]*MtwRankConfig)
}
